
BFC_WSG = {}; -- Initialize the BFC_WSG namespace

local BFC_WSG_REZ_INTERVAL = 32; -- rez wave every ~30s
local BFC_WSG_CLASS_SCAN_DELAY = 30; -- scan the scoreboard for class/level info every 30s

function BFC_WSG.Onload()
	-- Create the helper module and register it with the BFC system.
	local modobj = {
		Name = "Warsong Gulch Helper", -- A unique name, doesn't need to be the zone name
		Author = "Vallerius", -- Give yourself credit. Not used yet, might appear later in a management screen
		Version = "1.0", -- Also unused for now
		Type = "ZONE_HELPER", -- Must be ZONE_HELPER for per-zone helper objects
		SetActive = BFC_WSG.SetActive, -- Function to enable/disable the zone
		IsActiveZone = BFC_WSG.IsActiveZone, -- Function to determine if we're in a target zone
	};
	BFC_Common.RegisterHelperModule(modobj);
end

-- Called by the module dispatcher whenever we enter/leave an active zone
function BFC_WSG.SetActive(state)
	if(state == true) then
		BFC_WSG_Frame:RegisterEvent("AREA_SPIRIT_HEALER_IN_RANGE");
		BFC_WSG_Frame:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE"); 
		BFC_WSG_Frame:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE"); 
		BFC_WSG_Frame:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
		BFC_WSG_Frame:RegisterEvent("UPDATE_BATTLEFIELD_SCORE");
		
		-- Reset the mod state so that it's fresh every time
		BFC_WSG_RezTime:SetText(string.format(BFC_Strings.WSG.rezwavedefault));
		BFC_WSG.RezTimer = BFC_WSG_REZ_INTERVAL
		BFC_WSG.RezTimerActive = false;
		BFC_WSG.classScanTimer = 0;
		BFC_WSG.AlliancePlayers = {};
		BFC_WSG.HordePlayers = {};
		
		-- The update function will be called every frame, even if the WSG helper frame isn't visible.
		BFC_Common.RegisterUpdateFunction("BFC_WSG", BFC_WSG.OnUpdate);
		
		-- Zone-specific comms menu
		BFC_Radio.SetLocalMenuItems(BFC_Strings.CommsMenu_WSG);
		
		-- Receive rez time notifications
		BFC_Comms.RegisterMessage("WSG", "REZ", BFC_WSG.HandleRezMessage);
		BFC_WSG_Frame:Show();
	else
		BFC_WSG_Frame:UnregisterEvent("AREA_SPIRIT_HEALER_IN_RANGE");
		BFC_WSG_Frame:UnregisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE"); 
		BFC_WSG_Frame:UnregisterEvent("CHAT_MSG_BG_SYSTEM_HORDE"); 
		BFC_WSG_Frame:UnregisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
		BFC_WSG_Frame:UnregisterEvent("UPDATE_BATTLEFIELD_SCORE");
		
		BFC_Comms.UnregisterMessage("WSG", "REZ");
		BFC_Radio.SetLocalMenuItems(nil);
		
		-- No need for the update when we're not active.
		BFC_Common.UnregisterUpdateFunction("BFC_WSG");
		BFC_WSG_Frame:Hide();
	end
end

-- Called by the module dispatcher to determine if we're in an active zone
function BFC_WSG.IsActiveZone(zone)
	if(zone == BFC_Strings.WSG.zone) then
		return true;
	else
		return false;
	end
end

-- Called by the common OnUpdate handler (registered/unregistered in SetActive)
function BFC_WSG.OnUpdate(elapsed)
	if(BFC_WSG.RezTimerActive) then
		BFC_WSG.LastTime = BFC_WSG.CurTime;
		BFC_WSG.CurTime = GetTime();
		
		
		BFC_WSG.RezTimer = BFC_WSG.RezTimer - (BFC_WSG.CurTime - BFC_WSG.LastTime);
		if(BFC_WSG.RezTimer < 0) then
			BFC_WSG.RezTimer = BFC_WSG.RezTimer + BFC_WSG_REZ_INTERVAL;
		end
		
		BFC_WSG_RezTime:SetText(string.format(BFC_Strings.WSG.rezwave, math.ceil(BFC_WSG.RezTimer)));
	end
	
	-- Request new scoreboard info periodically for class/level details
	if ( BFC_WSG.classScanTimer < 0 ) then
		BFC_WSG.classScanTimer = BFC_WSG_CLASS_SCAN_DELAY;
		RequestBattlefieldScoreData();
	else
		BFC_WSG.classScanTimer = BFC_WSG.classScanTimer - elapsed;
	end
end


function BFC_WSG.HandleRezMessage(player, timeleft)
	BFC.Log(BFC.LOG_DEBUG, "Syncing rez timer to " .. timeleft[1]);
	BFC_WSG.SyncTimer(timeleft[1]);
end


-- Set the rez timer to a new value
function BFC_WSG.SyncTimer(secondsLeft)
	BFC_WSG.CurTime = GetTime();
	BFC_WSG.RezTimer = secondsLeft;
	BFC_WSG.RezTimerActive = true;
end


-- Your standard event handler
function BFC_WSG.OnEvent()
	if(event == "AREA_SPIRIT_HEALER_IN_RANGE" and UnitIsGhost("player")) then
		local timeToRez = GetAreaSpiritHealerTime();
		BFC_WSG.SyncTimer(timeToRez);
		BFC_Comms.SendMessage("WSG", "REZ", timeToRez);
	elseif(event == "UPDATE_BATTLEFIELD_SCORE") then
		BFC_WSG.UpdateClassInfo();
	elseif(event == "CHAT_MSG_BG_SYSTEM_ALLIANCE" or event == "CHAT_MSG_BG_SYSTEM_HORDE" or event == "CHAT_MSG_BG_SYSTEM_NEUTRAL") then
		BFC_WSG.ParseFlagEvent(arg1);
	end
end


-- Grab the class info from the scoreboard
function BFC_WSG.UpdateClassInfo()
	local num = GetNumBattlefieldScores();
	for i=1,num do
		local name, killingBlows, honorKills, deaths, honorGained, faction, rank, race, class = GetBattlefieldScore(i);
		
		-- Strip the attached server name, if applicable
		local loc = strfind(name, "-");
		if(loc ~= nil) then
			loc = loc - 1;
		end
		name = strsub(name, 0, loc);

		if(faction == 1) then
			BFC_WSG.AlliancePlayers[name] = BFC_Util.GetRealClassName(class);
		else
			BFC_WSG.HordePlayers[name] = BFC_Util.GetRealClassName(class);
		end
	end
end


-- Figure out what happened to the flag
function BFC_WSG.ParseFlagEvent(text)
	local player = "";
	local faction = "";
	local flagStatus = "";
	local color = NORMAL_FONT_COLOR;
	
	--first get flag status
	if(strfind(text, BFC_Strings.WSG["picked"])) then
		flagStatus = BFC_Strings.WSG["picked"];
	elseif(strfind(text, BFC_Strings.WSG["captured"])) then
		flagStatus = BFC_Strings.WSG["captured"];
	elseif(strfind(text, BFC_Strings.WSG["wins"])) then
		flagStatus = BFC_Strings.WSG["wins"];
	elseif(strfind(text, BFC_Strings.WSG["returned"])) then
		flagStatus = BFC_Strings.WSG["returned"];
	elseif(strfind(text, BFC_Strings.WSG["dropped"])) then
		flagStatus = BFC_Strings.WSG["dropped"];
	else
		flagStatus = "";
	end
	
	--faction is going to be Alliance or Horde
	index = strfind(text, BFC_Strings.WSG["horde"]);
	if(strfind(text, BFC_Strings.WSG["horde"])) then
		faction = BFC_Strings.WSG["horde"];
	elseif(strfind(text, BFC_Strings.WSG["alliance"])) then
		faction = BFC_Strings.WSG["alliance"];
	end

	--get the player who has the flag, only if the status is picked
	if (flagStatus == BFC_Strings.WSG["picked"] ) then

		-- FIXME: strip this out to the localization file
		--to be added for localization (blatantly stolen from BGFlag)
		if ( GetLocale() == "frFR") then
			if (strfind(text, " par ")) then
				player = strsub(text, strfind(text, " par ")+5, strlen(text)-2);
			end
		elseif ( GetLocale() == "deDE") then
			if (strfind(text, BFC_Strings.WSG["picked"])) then
				_,_,player = strfind(text, "(%w+) hat die Flagge");
			end
		else
			if (strfind(text, " by ")) then
				player = strsub(text, strfind(text, " by ")+4, strlen(text)-1);
			end
		end
		
		
		
		-- Set the class color for the picking player
		if(faction == BFC_Strings.WSG["alliance"]) then
			if(BFC_WSG.HordePlayers[player]) then
				color = RAID_CLASS_COLORS[BFC_WSG.HordePlayers[player]];
			end
		elseif(faction == BFC_Strings.WSG["horde"]) then
			if(BFC_WSG.AlliancePlayers[player]) then
				color = RAID_CLASS_COLORS[BFC_WSG.AlliancePlayers[player]];
			end
		end
	elseif (flagStatus == BFC_Strings.WSG["dropped"] ) then
		player = BFC_Strings.WSG["dropped"];
	elseif (flagStatus == BFC_Strings.WSG["captured"] or flagStatus == BFC_Strings.WSG["returned"] or flagStatus == "") then
		player = BFC_Strings.WSG["atbase"];
	end
	
	if(faction == BFC_Strings.WSG["alliance"]) then
		BFC_WSG_AllianceCarrier:SetText(player);
		BFC_WSG_AllianceCarrier:SetTextColor(color.r, color.g, color.b);
	elseif (faction == BFC_Strings.WSG["horde"]) then
		BFC_WSG_HordeCarrier:SetText(player);
		BFC_WSG_HordeCarrier:SetTextColor(color.r, color.g, color.b);
	end
end

function BFC_WSG.TargetCarrier(carrier)
	if(carrier and carrier ~= "" and carrier ~= BFC_Strings.WSG["atbase"] and carrier ~= BFC_Strings.WSG["dropped"]) then
		TargetByName(carrier);
	end
end